/*"Gear Indicator": Simple OutGauge client for LFS
 * Author: Vladimir Kadlec
 *         vladimirkadlec@gmail.com
 *         http://www.fi.muni.cz/~xkadlec
 */
/*
This file is part of Gear Indicator.

Gear Indicator is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Gear Indicator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gear Indicator; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <time.h>
#include <math.h>

#include "lpt.h"
#include "log.h"
#include "outsim.h"

//r, N, 1, 2, 3, 4, 5, 6, 7, 8
//char SEG_N[10] = {129, 131, 192, 87, 213, 225, 181, 183, 196, 247};
//char SEG_DOT = 8;

typedef short _stdcall (*inpfuncPtr)(short portaddr);
typedef void _stdcall (*oupfuncPtr)(short portaddr, short data);
inpfuncPtr inp32;
oupfuncPtr oup32;

static clock_t last_t;
static int dot_state = 0;

int init_lpt (int base) {
HINSTANCE hLib;

    hLib = LoadLibrary("inpout32.dll");
     if (hLib == NULL) {
          error ("LoadLibrary Failed.\n");
          return (0);
     }
     inp32 = (inpfuncPtr) GetProcAddress(hLib, "Inp32");
     if (inp32 == NULL) {
          error ("GetProcAddress for Inp32 Failed.\n");
          return (0);
     }

     oup32 = (oupfuncPtr) GetProcAddress(hLib, "Out32");
     if (oup32 == NULL) {
          error ("GetProcAddress for Oup32 Failed.\n");
          return (0);
     }
     last_t = clock();
    /* while (1) {
        (oup32) (0x378+2, 1);
        Sleep (100);
        (oup32) (0x378+2, 0);
        Sleep (100);
     }*/
	return (1);
}

int display_off (Config *conf) {
    (oup32) (conf->lpt_base, conf->disp_off);
    (oup32) (conf->lpt_base+2, conf->ext_off);
    return (1);
}

int display_lpt (Config *conf, OutGaugePacket *packet) {
char o;
clock_t act_t;
word flags;
short lptext_fl[sizeof (PINS)] = {1, 2, 4, 8};
short res_fl = 0;
int i;
    if (packet->Gear < 9)
        o = conf->display[packet->Gear];
    else o = conf->disp_off;
    
    flags = packet->Flags;
    if (flags & OG_PITSPEED) {
        act_t = clock ();
        if ( abs (act_t - last_t) > conf->blink_time ) {
            dot_state = !dot_state;
            last_t = act_t;
        }
    }
    if ((flags & OG_SHIFTLIGHT) || ((dot_state != 0) && (flags & OG_PITSPEED)))
        o |= conf->shift_light;
    (oup32) (conf->lpt_base, o);

    for (i = 0; i < sizeof (PINS); i++) {
        if (conf->pin_func[i] == F_NONE)
            continue;

        if (conf->pin_func[i] == F_LOW_FUEL) {
            if (packet->Fuel >= conf->fuel_warning*0.01) {
                if (conf->pin_func_neg[i] != 0) {
                    res_fl |= lptext_fl[i];
                    continue;
                }
                continue;
            }
//            printf ("Low fuel %f\r", packet->Fuel);
            if (conf->pin_func_neg[i] == 0) {
                res_fl |= lptext_fl[i];
                continue;
            }
            continue;
        }
        if (conf->pin_lfs_flag[i] & flags) {
            if (conf->pin_func_neg[i] == 0) {
                res_fl |= lptext_fl[i];
                continue;
            }
            continue;
        }
        if (conf->pin_func_neg[i] != 0) {
            res_fl |= lptext_fl[i];
            continue;
        }
    }
    (oup32) (conf->lpt_base+2, res_fl);
    return (1);
}
